@@ -51,20 +51,5 @@ pthreads <- function () {
 
 
 
-TRANSFORM_PCT   <- 1L
-TRANSFORM_CLR   <- 2L
-TRANSFORM_CHORD <- 3L
 
-transform_pct <- function (counts, cpus = n_cpus()) {
-  .Call(C_transform, counts, TRANSFORM_PCT, cpus, NULL)
-}
-
-transform_clr <- function (counts, pseudocount = NULL, cpus = n_cpus()) {
-  if (is.null(pseudocount)) pseudocount <- min(counts[counts > 0])
-  .Call(C_transform, counts, TRANSFORM_CLR, cpus, pseudocount)
-}
-
-transform_chord <- function (counts, cpus = n_cpus()) {
-  .Call(C_transform, counts, TRANSFORM_CHORD, cpus, NULL)
-}
 
